"""dash-spa - Dash Single Page Application (SPA) Framework"""

from .spa_dependency import SpaDependency
from .spa_components import SpaComponents
from .spa import SinglePageApp
from .spa_blueprint import Blueprint


__version__ = '0.0.1'
__author__ = 'Steve Jones <jonesst2608@gmail.com>'
__all__ = []
